//////////////////////////////////////////////////////////////////////////////
// This file was originally written by Bradley S. Meyer.
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//!
//! \file
//! \brief A header file defining screening functions.
//!
////////////////////////////////////////////////////////////////////////////////

#ifndef MY_USER_SCREEN_H
#define MY_USER_SCREEN_H

#include <Libnucnet.h>

#include "nnt/wrappers.hpp"
#include "nnt/string_defs.h"

#include "user/network_utilities.h"

namespace my_user
{

//##############################################################################
// User-supplied data type.
//##############################################################################

typedef struct screening_data_t
{

  double F1, F2, F4, F5;
  double F22;

  double dYe, dYtot, dZschl, dL0, dH120, dH120M, dH120A;

  screening_data_t( nnt::Zone zone )
  {

    F1 = 1. / 3.;
    F2 = 2. / 3.;
    F4 = 4. / 3.;
    F5 = 5. / 3.;

    F22 = 0.624;

    dYe =
      user::compute_cluster_abundance_moment( zone, "", "z",  1. );

    dYtot =
      user::compute_cluster_abundance_moment( zone, "", "z",  0. );

    dZschl =
      sqrt( 
        user::compute_cluster_abundance_moment( zone, "", "z",  2. ) + dYe
      );

    dH120 =
      user::compute_cluster_abundance_moment( zone, "", "z", 1.58 );

    dL0 =
      1.88e8 *
      sqrt(
        zone.getProperty<double>( nnt::s_RHO ) /
        gsl_pow_3(
          zone.getProperty<double>( nnt::s_T9 ) *
          GSL_CONST_NUM_GIGA
        )
      );

    dH120M =
      dH120 * 0.38 * pow( dL0, 0.86 ) /
      ( pow( dZschl, 0.58 ) * pow( dYe, 0.28 ) );

    dH120A = F22 * pow( dYe, F1 ) * pow( dL0, F2 );

  }

} screening_data_t;

//##############################################################################
// Prototypes for user-supplied functions.
//##############################################################################

void
set_screening_function( nnt::Zone& zone );

screening_data_t
get_screening_data( nnt::Zone& zone );

void
screening_function(
  Libnucnet__Zone *,
  Libnucnet__Reaction *,
  double,
  double,
  double,
  double *,
  double *
);

void
reaction_screening_function(
  Libnucnet__Net *,
  Libnucnet__Reaction *,
  double,
  double,
  double,
  void *,
  double *,
  double *
);

double
pair_screening_function(
  double,
  double,
  double,
  unsigned int,
  unsigned int,
  unsigned int,
  unsigned int,
  void *
);

} // namespace my_user

#endif // MY_USER_SCREEN_H
